@@ -19,32 +19,59 @@
 //!
 //! // Initializes a Datadog instance
 //! Datadog::init(configuration);
+//! ```
 //!
-//! // Then you can use the macros exposed at the base level of the module
+//! Then you can use the macros exposed at the base level of the module.
+//! All macros accepts
+//! - a literal string or a path to a type that implements AsRef<str> as first argument.
+//! - zero or more arguments, separated by comma `,`, for the metrics that needs more data.
+//!     For exemple `count!` and `timing!` accepts a number while `service_check!` accepts a [ServiceStatus] and a [ServiceCheckOptions]
+//! - a list of tags (which is separated from the rest of the arguments by semicolon `;`) in the form of `"name" => "value"`
+//!
+//! ```
+//! # use prima_datadog::{*, configuration::PrimaConfiguration};
+//! # let configuration = PrimaConfiguration::new(
+//! #     "0.0.0.0:1234", // to address
+//! #     "0.0.0.0:0", // from address
+//! #     "service_name", // namespace for all metrics
+//! #     "production".parse().unwrap() // environment
+//! # );
+//! # Datadog::init(configuration);
 //! incr!("test");
-//! incr!("test"; "some" => "data");
-//! decr!("test");
+//! # incr!("test"; "some" => "data");
+//! # decr!("test");
 //! decr!("test"; "some" => "data");
 //! count!("test", 20);
 //! count!("test", 10; "some" => "data");
 //! time!("test", || { println!("expensive computation");});
 //! time!("test", || { println!("expensive computation");}; "some" => "data");
-//! timing!("test", 20);
+//! # timing!("test", 20);
 //! timing!("test", 20; "some" => "data");
-//! gauge!("test", "gauge value");
+//! # gauge!("test", "gauge value");
 //! gauge!("test", "gauge value"; "some" => "data");
-//! histogram!("test", "histogram value");
+//! # histogram!("test", "histogram value");
 //! histogram!("test", "histogram value"; "some" => "data");
-//! distribution!("test", "distribution value");
+//! # distribution!("test", "distribution value");
 //! distribution!("test", "distribution value"; "some" => "data");
-//! set!("test", "set value");
+//! # set!("test", "set value");
 //! set!("test", "set value"; "some" => "data");
-//!  
-//! // The first argument is the metric name. It accepts string literal (like the previous example)
-//! // or a type path that implements [AsRef] for `T: str`
+//! service_check!("test", ServiceStatus::OK);
+//! service_check!("test", ServiceStatus::OK, ServiceCheckOptions::default());
+//! # event!("test", "test event");
+//! event!("test", "test event"; "some" => "data");
+//! ```
 //!
-//! // custom metric, based on an enum type. It can really be whatever you want, //!
-//! // as long as it implements AsRef<str>
+//! This is an example of a custom metric, in this case based on an enum type, but it can really be whatever you want, as long as it implements AsRef<str>
+//!
+//! ```
+//! # use prima_datadog::{*, configuration::PrimaConfiguration};
+//! # let configuration = PrimaConfiguration::new(
+//! #     "0.0.0.0:1234", // to address
+//! #     "0.0.0.0:0", // from address
+//! #     "service_name", // namespace for all metrics
+//! #     "production".parse().unwrap() // environment
+//! # );
+//! # Datadog::init(configuration);
 //! enum Metric {
 //!     John,
 //!     Paul,
@@ -102,6 +129,13 @@ pub struct Datadog {
 impl Datadog {
     /// Initializes a Datadog instance with a struct that implements the [Configuration] trait
     pub fn init(configuration: impl Configuration) -> Result<(), Error> {
+        match INSTANCE.get() {
+            None => Self::do_init(configuration),
+            Some(_) => Err(Error::OnceCellAlreadyInitialized),
+        }
+    }
+
+    fn do_init(configuration: impl Configuration) -> Result<(), Error> {
         let dogstatsd_client_options = dogstatsd::Options::new(
             configuration.from_addr(),
             configuration.to_addr(),
@@ -224,6 +258,30 @@ impl Datadog {
         let tags: Vec<String> = tags.into_iter().chain(self.default_tags.clone()).collect();
         self.client.set(metric.as_ref(), value.as_ref(), tags);
     }
+
+    /// Report the status of a service
+    pub fn service_check(
+        &self,
+        metric: impl AsRef<str>,
+        value: ServiceStatus,
+        tags: impl IntoIterator<Item = String>,
+        options: Option<ServiceCheckOptions>,
+    ) {
+        let tags: Vec<String> = tags.into_iter().chain(self.default_tags.clone()).collect();
+        self.client
+            .service_check(metric.as_ref(), value, tags, options);
+    }
+
+    /// Send a custom event as a title and a body
+    pub fn event(
+        &self,
+        metric: impl AsRef<str>,
+        text: impl AsRef<str>,
+        tags: impl IntoIterator<Item = String>,
+    ) {
+        let tags: Vec<String> = tags.into_iter().chain(self.default_tags.clone()).collect();
+        self.client.event(metric.as_ref(), text.as_ref(), tags);
+    }
 }
 
 /// Increment a StatsD counter
@@ -470,3 +528,121 @@ macro_rules! set {
         }
     };
 }
+
+/// Report a value in a set
+#[macro_export]
+macro_rules! service_check {
+    // call with literal and status
+    ($stat:literal, $service_status:path) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, vec![], None);
+        }
+    };
+    ($stat:literal, $service_status:path, $options: ident) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, vec![], Some($options));
+        }
+    };
+    ($stat:literal, $service_status:path, $options: expr) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, vec![], Some($options));
+        }
+    };
+    // call with path, status and options
+    ($stat:path, $service_status:path) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, vec![], None);
+        }
+    };
+    ($stat:path, $service_status:path, $options: ident) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, vec![], Some($options));
+        }
+    };
+    ($stat:path, $service_status:path, $options: expr) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, vec![], Some($options));
+        }
+    };
+    // call with literal, status, options and tags
+    ($stat:literal, $service_status:path; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], None);
+        }
+    };
+    ($stat:literal, $service_status:path, $options: ident; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], Some($options));
+        }
+    };
+    ($stat:literal, $service_status:path, $options: expr; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat, $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], Some($options));
+        }
+    };
+    // call with path, status, options and tags
+    ($stat:path, $service_status:path; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], None);
+        }
+    };
+    ($stat:path, $service_status:path, $options: ident; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], Some($options));
+        }
+    };
+    ($stat:path, $service_status:path, $options: expr; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().service_check($stat.as_ref(), $service_status, std::vec![$(std::format!("{}:{}", $key, $value)), *], Some($options));
+        }
+    };
+}
+
+/// Send a custom event as a title and a body
+#[macro_export]
+macro_rules! event {
+    ($stat:literal, $text:literal) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().event($stat, $text, vec![]);
+        }
+    };
+    ($stat:path, $text:literal) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().event($stat.as_ref(), $text, vec![]);
+        }
+    };
+    ($stat:literal, $text:literal; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().event($stat, $text, std::vec![$(std::format!("{}:{}", $key, $value)), *]);
+        }
+    };
+    ($stat:path, $text:literal; $( $key:expr => $value:expr ), *) => {
+        if $crate::Datadog::global().is_reporting_enabled() {
+            $crate::Datadog::global().event($stat.as_ref(), $text, std::vec![$(std::format!("{}:{}", $key, $value)), *]);
+        }
+    };
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::configuration::{Environment, PrimaConfiguration};
+
+    #[test]
+    pub fn double_initialization() {
+        let datadog = Datadog::init(PrimaConfiguration::new(
+            "10.1.2.3:8125",
+            "127.0.0.1:9000",
+            "",
+            Environment::Dev,
+        ));
+        assert!(datadog.is_ok());
+        let datadog2 = Datadog::init(PrimaConfiguration::new(
+            "10.1.2.3:8125",
+            "127.0.0.1:9000",
+            "",
+            Environment::Production,
+        ));
+        assert!(datadog2.err().unwrap().is_once_cell_already_initialized());
+    }
+}
